/*!
 * \file      sx1262mbxcas-board.c
 *
 * \brief     Target board SX1262MBXCAS shield driver implementation
 *
 * \copyright Revised BSD License, see section \ref LICENSE.
 *
 * \code
 *                ______                              _
 *               / _____)             _              | |
 *              ( (____  _____ ____ _| |_ _____  ____| |__
 *               \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 *               _____) ) ____| | | || |_| ____( (___| | | |
 *              (______/|_____)_|_|_| \__)_____)\____)_| |_|
 *              (C)2013-2017 Semtech
 *
 * \endcode
 *
 * \author    Miguel Luis ( Semtech )
 *
 * \author    Gregory Cristian ( Semtech )
 */
#include <stdlib.h>
#include "project_config.h"
#include "delay.h"
#include "radio.h"
#include "LLCC68-board.h"
#include "HAL_SPI.h"
#include "stdio.h"

typedef struct __softTimer_s{
	uint8_t isRun;		//0ֹͣ1
	uint32_t delayMs;	//ʱҪ೤ʱ
	uint32_t startMs;	//ʱʼʱ
}SoftTimer_t;

static DioIrqHandler *dio1IrqCallback=NULL;	//¼DIO1Ļص
static uint32_t timer_count=0;
static SoftTimer_t txTimerHandle,rxTimerHandle;

/* ʼLLCC68ҪõGPIOʼ BUSY Ϊģʽ
 */
void LLCC68IoInit( void )
{
	uint8_t u8_ret=255;
	GPIO_InitTypeDef  GPIO_InitStructure;

	//ʼBUSYΪģʽ
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB, ENABLE);	    //ʹָ˿ʱ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IPU;//
	GPIO_InitStructure.GPIO_Pin = RADIO_DIO4_BUSY_PIN;
	GPIO_Init(RADIO_DIO4_BUSY_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	//⼸ûõΪģʽ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IN_FLOATING;//
	
	GPIO_InitStructure.GPIO_Pin = RADIO_DIO0_TXEN_PIN;
	GPIO_Init(RADIO_DIO0_TXEN_PORT, &GPIO_InitStructure);	//ʼGPIO

	GPIO_InitStructure.GPIO_Pin = RADIO_DIO2_PIN;
	GPIO_Init(RADIO_DIO2_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	GPIO_InitStructure.GPIO_Pin = RADIO_DIO3_PIN;
	GPIO_Init(RADIO_DIO3_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	GPIO_InitStructure.GPIO_Pin = RADIO_DIO5_RXEN_PIN;
	GPIO_Init(RADIO_DIO5_RXEN_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	u8_ret=Spi1Init();
	if(u8_ret){
		printf("spi1 init error\r\n");
	}else{
		printf("LLCC68IoInit init sucess\r\n");
	}
}

/* ʼ DIO1
 * DIO1Ϊⲿж(ش)һصΪ dioIrq ԭ void RadioOnDioIrq( void* context )
 */
void LLCC68IoIrqInit( DioIrqHandler dioIrq )
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;
 	NVIC_InitTypeDef NVIC_InitStructure;
	
	dio1IrqCallback=dioIrq;
	
	//gpio(DIO1)ʼΪģʽ
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB, ENABLE);	    //ʹָ˿ʱ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IPD;//
	GPIO_InitStructure.GPIO_Pin = RADIO_DIO1_PIN;
	GPIO_Init(RADIO_DIO1_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	//ж
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,ENABLE);	//ʹܸùʱ
	//GPIOE.2 жԼжϳʼ   ش
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOB,GPIO_PinSource11);
	EXTI_InitStructure.EXTI_Line=EXTI_Line11;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;	
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;	//ش
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);	 	//EXTI_InitStructָĲʼEXTIĴ

	//жȼ
	NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;			//ʹܰWK_UPڵⲿжͨ
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x02;	//ռȼ2 
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x03;					//ȼ3
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;								//ʹⲿжͨ
	NVIC_Init(&NVIC_InitStructure);
	EXTI_ClearITPendingBit(EXTI_Line11);  //LINE11ϵжϱ־λ 
}

//жϻصлص void LLCC68IoIrqInit( DioIrqHandler dioIrq ) ע dioIrq ص
void EXTI15_10_IRQHandler(void){
	//printf("enter irq\r\n");
	if(GPIO_ReadInputDataBit(RADIO_DIO1_PORT,RADIO_DIO1_PIN)){
		printf("DIO1 irq\r\n");
		if(NULL!=dio1IrqCallback){
			dio1IrqCallback(NULL);
		}
		EXTI_ClearITPendingBit(EXTI_Line11);//жϱ־λ
	}
}

void LLCC68IoDeInit( void )
{
    //GPIOȥʼ
}

//λ
void LLCC68Reset( void )
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	LLCC68DelayMs( 10 );
	
	//RST͵ƽ
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB, ENABLE);	    //ʹָ˿ʱ
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	 //IOٶΪ50MHzﲻôΣֱдٶ50MHZ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;	//
	GPIO_InitStructure.GPIO_Pin = RADIO_nRESET_PIN;
	GPIO_Init(RADIO_nRESET_PORT, &GPIO_InitStructure);	//ʼGPIO
	GPIO_WriteBit( RADIO_nRESET_PORT, RADIO_nRESET_PIN,Bit_RESET);	//RSTΪ͵ƽ
	
	LLCC68DelayMs( 20 );
	
	//RSTΪģʽ
	GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IPU;
	GPIO_Init(RADIO_nRESET_PORT, &GPIO_InitStructure);	//ʼGPIO
	
	LLCC68DelayMs( 10 );
}

//ȡBusyŵƽ״̬
void LLCC68WaitOnBusy( void )
{
	uint32_t u32_count=0;
	
	while( GPIO_ReadInputDataBit(RADIO_DIO4_BUSY_PORT,RADIO_DIO4_BUSY_PIN) == 1 ){
		if(u32_count++>1000){
			printf("wait busy pin timeout\r\n");
			u32_count=0;
		}
		LLCC68DelayMs(1);
	}
}

//SPINSSŵƽ0͵ƽߵƽ
void LLCC68SetNss(uint8_t lev ){
	if(lev){
		GPIO_SetBits(RADIO_NSS_PORT,RADIO_NSS_PIN);	//ߵƽ
	}else{
		GPIO_ResetBits(RADIO_NSS_PORT,RADIO_NSS_PIN);	//͵ƽ
	}
}

//spiһֽڵ
uint8_t LLCC68SpiInOut(uint8_t data){
	return HALSpi1InOut(data);
}

//ƵǷҪҪжֱӷtrue
bool LLCC68CheckRfFrequency( uint32_t frequency )
{
    // Implement check. Currently all frequencies are supported
    return true;
}

//ʱ
void LLCC68DelayMs(uint32_t ms){
	delay_ms(ms);
}

//tx/rxʱ

//ʱ3жϷ
//txʱҪص RadioOnTxTimeoutIrq
//rxʱҪص RadioOnRxTimeoutIrq
void TIM3_IRQHandler(void)   //TIM3ж
{
	uint32_t diffMs=0;
	
	if (TIM_GetITStatus(TIM3, TIM_IT_Update) != RESET){
		timer_count++;
		TIM_ClearITPendingBit(TIM3, TIM_IT_Update  );  //TIMxжϱ־
		
		//printf("timer_count=%d\r\n",timer_count);
		
		//txʱ
		if(txTimerHandle.isRun){
			if(timer_count>=txTimerHandle.startMs){
				diffMs=timer_count-txTimerHandle.startMs;
			}else{//
				diffMs=0xffffffff - txTimerHandle.startMs + timer_count;
			}
			if(diffMs>txTimerHandle.delayMs){
				//ʱ
				//printf("----------------- tx timer irq ---------------\r\n");
				LLCC68TxTimerStop();
				txTimerHandle.startMs=timer_count;
				RadioOnTxTimeoutIrq(NULL);	//txʱҪص RadioOnTxTimeoutIrq
			}
		}
		
		//rxʱ
		if(rxTimerHandle.isRun){
			if(timer_count>=rxTimerHandle.startMs){
				diffMs=timer_count-rxTimerHandle.startMs;
			}else{//
				diffMs=0xffffffff - rxTimerHandle.startMs + timer_count;
			}
			if(diffMs>rxTimerHandle.delayMs){
				//ʱ
				//printf("----------------- rx timer irq ---------------\r\n");
				LLCC68RxTimerStop();
				rxTimerHandle.startMs=timer_count;
				RadioOnRxTimeoutIrq(NULL);	//rxʱҪص RadioOnRxTimeoutIrq
			}
		}
	}
}

//ʼʱ(TIM3ʱģmsʱ)
//txʱҪص RadioOnTxTimeoutIrq
//rxʱҪص RadioOnRxTimeoutIrq
void LLCC68TimerInit(void){
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	//ʹTIM3
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); //ʱʹ
 
	//TIM_ITConfig(TIM3,TIM_IT_Update,ENABLE ); //ʹָTIM3ж,ж

	//жȼNVIC
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;  //TIM3ж
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;  //ռȼ0
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;  //ȼ3
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE; //IRQͨʹ
	NVIC_Init(&NVIC_InitStructure);  //ʼNVICĴ
	
	//ʱTIM3ʼ
	TIM_TimeBaseStructure.TIM_Period = 10; //һ¼װԶװؼĴڵֵ	
	TIM_TimeBaseStructure.TIM_Prescaler =SystemCoreClock/10000; //ΪTIMxʱƵʳԤƵֵ(1s10000ΣҲ10.1ms)
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1; //ʱӷָ:TDTS = Tck_tim
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;  //TIMϼģʽ
	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure); //ָĲʼTIMxʱλ
	
	//رʱ
	txTimerHandle.isRun=0;
	rxTimerHandle.isRun=0;
	
	TIM_Cmd(TIM3, ENABLE);  //ʹTIMx
	TIM_ClearITPendingBit(TIM3, TIM_IT_Update  );  //TIMxжϱ־
	TIM_ITConfig(TIM3,TIM_IT_Update,ENABLE ); //ʹָTIM2ж,ж
}

void LLCC68SetTxTimerValue(uint32_t nMs){
	printf("[%s()-%d]set timer out %d ms\r\n",__func__,__LINE__,nMs);
	txTimerHandle.delayMs=nMs;
}

void LLCC68TxTimerStart(void){
	printf("[%s()-%d]start timer\r\n",__func__,__LINE__);
	txTimerHandle.startMs=timer_count;
	txTimerHandle.isRun=1;
}

void LLCC68TxTimerStop(void){
	printf("[%s()-%d]stop timer\r\n",__func__,__LINE__);
	txTimerHandle.isRun=0;
}

void LLCC68SetRxTimerValue(uint32_t nMs){
	printf("[%s()-%d]set timer out %d ms\r\n",__func__,__LINE__,nMs);
	rxTimerHandle.delayMs=nMs;
}

void LLCC68RxTimerStart(void){
	printf("[%s()-%d]start timer\r\n",__func__,__LINE__);
	rxTimerHandle.startMs=timer_count;
	rxTimerHandle.isRun=1;
}

void LLCC68RxTimerStop(void){
	printf("[%s()-%d]stop timer\r\n",__func__,__LINE__);
	rxTimerHandle.isRun=0;
}
